@@ -1,4 +1,4 @@
-/* Copyright (C) 2003 Thorsten Kukuk
+/* Copyright (C) 2003, 2005 Thorsten Kukuk
    Author: Thorsten Kukuk <kukuk@suse.de>
 
    This program is free software; you can redistribute it and/or modify
@@ -372,8 +372,28 @@ write_group_data (group_t *data, int is_locked)
 	      retval = -1;
 	      goto error_group;
 	    }
-          fchmod (newgf_fd, group_stat.st_mode);
-          fchown (newgf_fd, group_stat.st_uid, group_stat.st_gid);
+          if (fchmod (newgf_fd, group_stat.st_mode) < 0)
+            {
+              fprintf (stderr,
+                       _("Cannot change mode for `%s': %s\n"),
+                       group_tmp, strerror (errno));
+              fclose (oldgf);
+              close (newgf_fd);
+              unlink (group_tmp);
+              retval = -1;
+              goto error_group;
+            }
+          if (fchown (newgf_fd, group_stat.st_uid, group_stat.st_gid) < 0)
+	                {
+              fprintf (stderr,
+                       _("Cannot change owner/group for `%s': %s\n"),
+                       group_tmp, strerror (errno));
+              fclose (oldgf);
+              close (newgf_fd);
+              unlink (group_tmp);
+              retval = -1;
+              goto error_group;
+            }
 	  if (copy_xattr (group_orig, group_tmp) != 0)
 	    {
 	      fclose (oldgf);
@@ -498,197 +518,20 @@ write_group_data (group_t *data, int is_locked)
 	      goto error_group;
 	    }
 	  unlink (group_old);
-	  link (group_orig, group_old);
-	  rename (group_tmp, group_orig);
+	  if (link (group_orig, group_old) < 0)
+	    fprintf (stderr,
+                     _("Warning: cannot create backup of group file: %m\n"));
+	  if (rename (group_tmp, group_orig) < 0)
+	    {
+	      fprintf (stderr,
+                       _("Error while renaming temporary group file: %m\n"));
+              retval = -1;
+              goto error_group;
+            }
 	error_group:
 	  unlink (group_tmp);
 	}
 
-#if 0
-      /* XXX gshadow is not supported yet.  */
-
-      /* If we use gshadow data, check if we need to change here
-	 something, too. But do this only, if there no error occured
-	 before.  */
-      if (data->use_gshadow && retval == 0)
-        {
-	  const char *file_tmp = "/gshadow.tmpXXXXXX";
-	  char *gshadow_tmp = alloca (strlen (files_etc_dir) + strlen (file_tmp) + 1);
-	  char *gshadow_orig = alloca (strlen (files_etc_dir) + 9);
-	  char *gshadow_old = alloca (strlen (files_etc_dir) + 13);
-          struct stat group_stat;
-          struct spwd *sp; /* shadow struct obtained from fgetspent() */
-          FILE *oldgf, *newgf;
-          int gotit, newgf_fd;
-	  char *cp;
-
-	  cp = stpcpy (gshadow_tmp, files_etc_dir);
-	  strcpy (cp, file_tmp);
-	  cp = stpcpy (gshadow_orig, files_etc_dir);
-	  strcpy (cp, "/gshadow");
-	  cp = stpcpy (gshadow_old, gshadow_orig);
-	  strcpy (cp, ".old");
-
-          /* Open the gshadow file for reading. We can't use getspent and
-             friends here, because they go through the YP maps, too. */
-          if ((oldgf = fopen (gshadow_orig, "r")) == NULL)
-            {
-	      fprintf (stderr, _("Can't open %s: %m\n"), gshadow_orig);
-	      retval = -1;
-              goto error_gshadow;
-            }
-          if (fstat (fileno (oldgf), &group_stat) < 0)
-            {
-	      fprintf (stderr, _("Can't stat %s: %m\n"), gshadow_orig);
-              fclose (oldgf);
-	      retval = -1;
-              goto error_gshadow;
-            }
-
-#ifdef WITH_SELINUX
-	  security_context_t prev_context;
-	  if (set_default_context (gshadow_orig, &prev_context) < 0)
-	    {
-	      fclose (oldgf);
-	      retval = -1;
-	      goto error_gshadow;
-	    }
-#endif
-          /* Open a temp gshadow file */
-          newgf_fd = mkstemp (gshadow_tmp);
-#ifdef WITH_SELINUX
-	  restore_default_context (prev_context);
-#endif
-          if (newgf_fd == -1)
-            {
-	      fprintf (stderr, _("Can't create %s temp file: %m\n"),
-		       gshadow_orig);
-              fclose (oldgf);
-	      retval = -1;
-              goto error_gshadow;
-            }
-          fchmod (newgf_fd, group_stat.st_mode);
-          fchown (newgf_fd, group_stat.st_uid, group_stat.st_gid);
-	  if (copy_xattr (gshadow_orig, gshadow_tmp) != 0)
-	    {
-	      fclose (oldgf);
-	      close (newgf_fd);
-	      unlink (gshadow_tmp);
-	      retval = -1;
-	      goto error_group;
-	    }
-          newgf = fdopen (newgf_fd, "w+");
-          if (newgf == NULL)
-            {
-	      fprintf (stderr, _("Can't open %s: %m\n"), gshadow_tmp);
-              fclose (oldgf);
-              close (newgf_fd);
-	      retval = -1;
-              goto error_gshadow;
-            }
-
-          gotit = 0;
-
-          /* Loop over all group entries */
-          while ((sp = fgetspent (oldgf)) != NULL)
-            {
-	      if (data->new_group_entry)
-		{
-		  /* insert the new user before we find a +/- character
-		     or an old entry from a deleted user. */
-		  if (!gotit &&
-		      (sp->sp_namp[0] == '+' || sp->sp_namp[0] == '-' ||
-		       strcmp (sp->sp_namp, data->sp.sp_namp) == 0))
-		    {
-		      /* write the group entry to tmp file */
-		      if (putspent (&data->sp, newgf) < 0)
-			{
-			  fprintf (stderr, _("\
-Error while writing temporary gshadow file: %m\n"));
-			  fclose (oldgf);
-			  fclose (newgf);
-			  retval = -1;
-			  goto error_gshadow;
-			}
-		      gotit = 1;
-		    }
-		}
-	      else
-              /* check if this is the uid we want to change. A few
-                 sanity checks added for consistency. */
-              if (!gotit && strcmp (data->gr.gr_name, sp->sp_namp) == 0)
-                {
-		  if (data->newpassword)
-		    sp->sp_pwdp = data->newpassword;
-
-		  if (data->sp_changed)
-		    {
-		      /* if they have changed, they are different. If not,
-			 they are equal, so copying them doesn't matter.  */
-		      sp->sp_max = data->spn.sp_max;
-		      sp->sp_min = data->spn.sp_min;
-		      sp->sp_warn = data->spn.sp_warn;
-		      sp->sp_inact = data->spn.sp_inact;
-		      sp->sp_lstchg = data->spn.sp_lstchg;
-		      sp->sp_expire = data->spn.sp_expire;
-		    }
-
-                  gotit = 1;
-                }
-
-	      /* Write the entry only if we don't create a new
-		 account or if this is not a duplicate of the new
-		 user account.  */
-	      if (data->new_user == 0 ||
-		  strcmp (sp->sp_namp, data->sp.sp_namp) != 0)
-		{
-		  /* write the group entry to tmp file */
-		  if (putspent (sp, newgf) < 0)
-		    {
-		      fprintf (stderr, _("\
-Error while writing temporary gshadow file: %m\n"));
-		      fclose (oldgf);
-		      fclose (newgf);
-		      retval = -1;
-		      goto error_gshadow;
-		    }
-		}
-            }
-	  if (data->new_user && !gotit)
-	    {
-	      /* Add the new user at the end. */
-	      if (putspent (&data->sp, newgf) < 0)
-		{
-		  fprintf (stderr, _("\
-Error while writing temporary gshadow file: %m\n"));
-		  fclose (oldgf);
-		  fclose (newgf);
-		  retval = -1;
-		  goto error_gshadow;
-		}
-	    }
-	  if (fclose (oldgf) != 0)
-	    {
-	      fprintf (stderr, _("Error while closing old gshadow file: %m\n"));
-	      fclose (newgf);
-	      retval = -1;
-	      goto error_group;
-	    }
-	  if (fclose (newgf) != 0)
-	    {
-	      fprintf (stderr, _("\
-Error while closing temporary gshadow file: %m\n"));
-	      retval = -1;
-	      goto error_group;
-	    }
-          unlink (gshadow_old);
-          link (gshadow_orig, gshadow_old);
-          rename (gshadow_tmp, gshadow_orig);
-        error_gshadow:
-          unlink (gshadow_tmp);
-        }
-#endif
-
       if (!is_locked)
 	ulckpwdf ();
     }
