@@ -1,4 +1,4 @@
-/* Copyright (C) 2004 Thorsten Kukuk
+/* Copyright (C) 2004, 2005 Thorsten Kukuk
    Author: Thorsten Kukuk <kukuk@suse.de>
 
    This program is free software; you can redistribute it and/or modify
@@ -74,12 +74,12 @@ register_slp (int port, int slp_timeout, const char *slp_descr)
   SLPError err;
   SLPError callbackerr;
   SLPHandle hslp;
-  char *attr = "";
+  char *attr;
   int timeout;
 
   saved_port = port;
   saved_timeout = slp_timeout;
-  if (saved_descr == NULL)
+  if (saved_descr == NULL && slp_descr != NULL)
     saved_descr = strdup (slp_descr);
 
   if (url != NULL)
@@ -116,11 +116,16 @@ register_slp (int port, int slp_timeout, const char *slp_descr)
       return -1;
     }
 
-  if (asprintf (&attr, "(description=%s)", slp_descr) < 0)
+  if (slp_descr)
     {
-      dbg_log ("Out of memory");
-      return -1;
+      if (asprintf (&attr, "(description=%s)", slp_descr) < 0)
+	{
+	  dbg_log ("Out of memory");
+	  return -1;
+	}
     }
+  else
+    attr = strdup ("");
 
   err = SLPOpen ("en", SLP_FALSE, &hslp);
   if(err != SLP_OK)
@@ -136,6 +141,8 @@ register_slp (int port, int slp_timeout, const char *slp_descr)
 		pwdutilsSLPRegReport,
 		&callbackerr);
 
+  free (attr);
+
   /* err may contain an error code that occurred as the slp library    */
   /* _prepared_ to make the call.                                     */
   if ((err != SLP_OK) || (callbackerr != SLP_OK))
