@@ -478,8 +478,28 @@ write_user_data (user_t *data, int is_locked)
 	      retval = -1;
 	      goto error_passwd;
 	    }
-          fchmod (newpf_fd, passwd_stat.st_mode);
-          fchown (newpf_fd, passwd_stat.st_uid, passwd_stat.st_gid);
+          if (fchmod (newpf_fd, passwd_stat.st_mode) < 0)
+	    {
+	      fprintf (stderr,
+		       _("Cannot change mode for `%s': %s\n"),
+		       passwd_tmp, strerror (errno));
+	      fclose (oldpf);
+	      close (newpf_fd);
+	      unlink (passwd_tmp);
+	      retval = -1;
+	      goto error_passwd;
+	    }
+          if (fchown (newpf_fd, passwd_stat.st_uid, passwd_stat.st_gid) < 0)
+	    {
+	      fprintf (stderr,
+		       _("Cannot change owner/group for `%s': %s\n"),
+		       passwd_tmp, strerror (errno));
+	      fclose (oldpf);
+	      close (newpf_fd);
+	      unlink (passwd_tmp);
+	      retval = -1;
+	      goto error_passwd;
+	    }
 	  if (copy_xattr (passwd_orig, passwd_tmp) != 0)
 	    {
 	      fclose (oldpf);
@@ -612,8 +632,16 @@ write_user_data (user_t *data, int is_locked)
 	      goto error_passwd;
 	    }
 	  unlink (passwd_old);
-	  link (passwd_orig, passwd_old);
-	  rename (passwd_tmp, passwd_orig);
+	  if (link (passwd_orig, passwd_old) < 0)
+	    fprintf (stderr,
+		     _("Warning: cannot create backup of passwd file: %m\n"));
+	  if (rename (passwd_tmp, passwd_orig) < 0)
+	    {
+	      fprintf (stderr,
+		       _("Error while renaming temporary password file: %m\n"));
+	      retval = -1;
+	      goto error_passwd;
+	    }
 	error_passwd:
 	  unlink (passwd_tmp);
 	}
@@ -682,8 +710,28 @@ write_user_data (user_t *data, int is_locked)
 	      retval = -1;
               goto error_shadow;
             }
-          fchmod (newpf_fd, shadow_stat.st_mode);
-          fchown (newpf_fd, shadow_stat.st_uid, shadow_stat.st_gid);
+          if (fchmod (newpf_fd, shadow_stat.st_mode) < 0)
+	    {
+	      fprintf (stderr,
+		       _("Cannot change mode for `%s': %s\n"),
+		       shadow_tmp, strerror (errno));
+	      fclose (oldpf);
+	      close (newpf_fd);
+	      unlink (shadow_tmp);
+	      retval = -1;
+	      goto error_shadow;
+	    }
+          if (fchown (newpf_fd, shadow_stat.st_uid, shadow_stat.st_gid) < 0)
+	    {
+	      fprintf (stderr,
+		       _("Cannot change owner/group for `%s': %s\n"),
+		       shadow_tmp, strerror (errno));
+	      fclose (oldpf);
+	      close (newpf_fd);
+	      unlink (shadow_tmp);
+	      retval = -1;
+	      goto error_shadow;
+	    }
 	  if (copy_xattr (shadow_orig, shadow_tmp) != 0)
 	    {
 	      fclose (oldpf);
@@ -813,8 +861,15 @@ Error while closing temporary shadow file: %m\n"));
 	      goto error_shadow;
 	    }
           unlink (shadow_old);
-          link (shadow_orig, shadow_old);
-          rename (shadow_tmp, shadow_orig);
+	  if (link (shadow_orig, shadow_old) < 0)
+	    fprintf (stderr, _("Warning: cannot create backup of shadow file: %m\n"));
+          if (rename (shadow_tmp, shadow_orig) < 0)
+	    {
+	      fprintf (stderr,
+		       _("Error while renaming temporary shadow file: %m\n"));
+	      retval = -1;
+	      goto error_shadow;
+	    }
         error_shadow:
           unlink (shadow_tmp);
         }
