@@ -1,5 +1,5 @@
-/* Copyright (C) 2004 Thorsten Kukuk
-   Author: Thorsten Kukuk <kukuk@suse.de>
+/* Copyright (C) 2004,2005 Thorsten Kukuk
+   Author: Thorsten Kukuk <kukuk@thkukuk.de>
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License version 2 as
