@@ -22,11 +22,11 @@
 #define _GNU_SOURCE
 
 #include <pwd.h>
+#include <time.h>
 #include <ctype.h>
 #include <errno.h>
 #include <stdio.h>
 #include <string.h>
-#include <syslog.h>
 #include <signal.h>
 #include <unistd.h>
 #include <getopt.h>
@@ -35,8 +35,6 @@
 #include <libintl.h>
 #include <sys/stat.h>
 #include <sys/resource.h>
-#include <security/pam_appl.h>
-#include <security/pam_misc.h>
 #ifdef HAVE_LIBNSCD_H
 #include <libnscd.h>
 #endif
@@ -379,7 +377,7 @@ main (int argc, char *argv[])
           print_help (program);
           return 0;
         case 'v':
-          print_version (program, "2004");
+          print_version (program, "2005");
           return 0;
         case 'u':
           print_usage (stdout, program);
@@ -415,7 +413,7 @@ main (int argc, char *argv[])
       struct passwd *pw;
       char *arg_user;
 
-      /* Determine our own user name for PAM authentication.  */
+      /* Determine our own user name for authentication.  */
       while (getpwuid_r (uid, &resultbuf, buffer, buflen, &pw) != 0
 	     && errno == ERANGE)
 	{
@@ -539,10 +537,9 @@ main (int argc, char *argv[])
 	  return E_NOPERM;
 	}
 
-      if (setgid (getgid ()) || setuid (getuid ()))
+      if (setgid (getgid ()) || setuid (uid))
 	{
-	  syslog (LOG_ERR, "%s: Failed to drop privileges: %s\n",
-		  program, strerror (errno));
+	  sec_log (program, MSG_DROP_PRIVILEGE_FAILED, errno, uid);
 	  fprintf (stderr, _("%s: Failed to drop privileges: %s\n"),
 		   program, strerror (errno));
 	  return E_FAILURE;
