@@ -1,4 +1,4 @@
-/* Copyright (C) 2002, 2003, 2004 Thorsten Kukuk
+/* Copyright (C) 2002, 2003, 2004, 2005 Thorsten Kukuk
    Author: Thorsten Kukuk <kukuk@suse.de>
 
    This program is free software; you can redistribute it and/or modify
@@ -26,6 +26,8 @@
 #include <ctype.h>
 #include <errno.h>
 #include <stdio.h>
+#include <syslog.h>
+#include <string.h>
 #include <wctype.h>
 #include <signal.h>
 #include <unistd.h>
@@ -33,8 +35,6 @@
 #include <shadow.h>
 #include <locale.h>
 #include <libintl.h>
-#include <security/pam_appl.h>
-#include <security/pam_misc.h>
 #ifdef WITH_SELINUX
 #include <selinux/selinux.h>
 #include <selinux/av_permissions.h>
@@ -492,7 +492,7 @@ main (int argc, char *argv[])
 	    print_help_util (program);
           return 0;
         case 'v':
-          print_version (program, "2004");
+          print_version (program, "2005");
           return 0;
         case 'u':
 	  if (shadow_chfn)
@@ -522,7 +522,7 @@ main (int argc, char *argv[])
     struct passwd resultbuf;
     struct passwd *pw;
 
-    /* Determine our own user name for PAM authentication.  */
+    /* Determine our own user name for authentication.  */
     while (getpwuid_r (uid, &resultbuf, buffer, buflen, &pw) != 0
 	   && errno == ERANGE)
       {
