@@ -1,4 +1,4 @@
-/* Copyright (C) 2003, 2004 Thorsten Kukuk
+/* Copyright (C) 2003, 2004, 2005 Thorsten Kukuk
    Author: Thorsten Kukuk <kukuk@suse.de>
 
    This program is free software; you can redistribute it and/or modify
@@ -26,13 +26,13 @@
 #include <stdio.h>
 #include <string.h>
 #include <unistd.h>
-#include <syslog.h>
 #include <getopt.h>
 #include <locale.h>
 #include <libintl.h>
 
 #include "public.h"
 #include "utf8conv.h"
+#include "logging.h"
 #include "error_codes.h"
 
 #ifndef _
@@ -202,7 +202,7 @@ main (int argc, char *argv[])
   bindtextdomain(PACKAGE, LOCALEDIR);
   textdomain(PACKAGE);
 
-  openlog (program, LOG_PID, LOG_AUTHPRIV);
+  open_sec_log (program);
 
   while (1)
     {
@@ -233,7 +233,7 @@ main (int argc, char *argv[])
           print_help (program);
           return 0;
         case 'v':
-          print_version (program, "2004");
+          print_version (program, "2005");
           return 0;
         case 'u':
           print_usage (stdout, program);
@@ -305,10 +305,9 @@ main (int argc, char *argv[])
       fputs ("\n", stdout);
 
       /* drop privilegs.  */
-      if (setgid (getgid ()) || setuid (getuid ()))
+      if (setgid (getgid ()) || setuid (uid))
         {
-          syslog (LOG_ERR, "%s: Failed to drop privileges: %s\n",
-                  program, strerror (errno));
+	  sec_log (program, MSG_DROP_PRIVILEGE_FAILED, errno, uid);
           fprintf (stderr, _("%s: Failed to drop privileges: %s\n"),
                    program, strerror (errno));
           return E_FAILURE;
