@@ -1,4 +1,4 @@
-/* Copyright (C) 2004 Thorsten Kukuk
+/* Copyright (C) 2004, 2005 Thorsten Kukuk
    Author: Thorsten Kukuk <kukuk@suse.de>
 
    This program is free software; you can redistribute it and/or modify
@@ -26,7 +26,6 @@
 #include <stdio.h>
 #include <fcntl.h>
 #include <string.h>
-#include <syslog.h>
 #include <unistd.h>
 #include <signal.h>
 #include <getopt.h>
@@ -93,8 +92,6 @@ main (int argc, char *argv[])
   /* determine name of binary, which specifies edit mode.  */
   program = ((cp = strrchr (*argv, '/')) ? cp + 1 : *argv);
 
-  openlog (program, LOG_PID, LOG_AUTHPRIV);
-
   while (1)
     {
       int c;
@@ -125,7 +122,7 @@ main (int argc, char *argv[])
 	  print_help (program);
 	  return 0;
         case 'v':
-          print_version (program, "2004");
+          print_version (program, "2005");
           return 0;
         case 'u':
           print_usage (stdout, program);
@@ -363,7 +360,8 @@ main (int argc, char *argv[])
 		   pw->pw_dir, strerror (errno));
 	  if (!getlogindefs_bool ("DEFAULT_HOME", 1))
 	    return E_FAILURE;
-	  chdir ("/");
+	  if (chdir ("/") < 0)
+	    return E_FAILURE;
 	}
 
       term = getenv("TERM");
