@@ -379,11 +379,15 @@ query_slp (HANDLE char **hostp, char **portp, char **descrp)
 
       while ((choice < 1) || (choice > i))
 	{
-	  printf ("Enter number of choice: ");
+	  char *cp;
+	  printf (_("Enter number of choice [1-%d]: "), i);
 	  fflush (stdin);
-	  fgets (response, sizeof (response), stdin);
+	  cp = fgets (response, sizeof (response), stdin);
 	  fflush (stdin);
-	  choice = strtol (response, NULL, 10);
+	  if (cp == NULL)
+	    choice = 0;
+	  else
+	    choice = strtol (response, NULL, 10);
 	}
       printf ("\n");
       for (i = 0; i < (choice - 1); i++)
