@@ -98,7 +98,7 @@ print_version (const char *program)
 Copyright (C) %s Thorsten Kukuk.\n\
 This is free software; see the source for copying conditions.  There is NO\n\
 warranty; not even for MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.\n\
-"), "2004");
+"), "2005");
   /* fprintf (stdout, _("Written by %s.\n"), "Thorsten Kukuk"); */
 }
 
@@ -611,9 +611,10 @@ sane_locale (const char *name)
 static int
 #ifdef USE_GNUTLS
 handle_request (gnutls_session ssl, request_header *req, char *locale,
-		char *username)
+		const char *username, const char * program)
 #else
-handle_request (SSL *ssl, request_header *req, char *locale, char *username)
+handle_request (SSL *ssl, request_header *req, char *locale,
+		const char *username, const char *program)
 #endif
 {
   const struct pam_conv conv = {
@@ -668,6 +669,8 @@ cannot handle request version %d; current version is %d"),
 	 a failed.  */
       pam_authenticate (pamh, flags);
       pam_end (pamh, PAM_SUCCESS);
+      /* after using PAM we have to reset openlog data */
+      openlog (program, LOG_CONS | LOG_ODELAY, LOG_DAEMON);
       sleep (getlogindefs_num ("FAIL_DELAY", 1));
       send_string (ssl, ERROR_MSG, "Authentication failure");
       retval = E_FAILURE;
@@ -689,6 +692,8 @@ cannot handle request version %d; current version is %d"),
       ret = pam_start ("rpasswd", account, &conv, &pamh);
       if (ret != PAM_SUCCESS)
 	{
+	  /* after using PAM we have to reset openlog data */
+	  openlog (program, LOG_CONS | LOG_ODELAY, LOG_DAEMON);
 	  dbg_log ("Couldn't initialize PAM: %s", pam_strerror (pamh, ret));
 	  retval = E_PAM_ERROR;
 	  goto send_finish;
@@ -708,6 +713,9 @@ cannot handle request version %d; current version is %d"),
       ret = pam_authenticate (pamh, flags);
       if (ret != PAM_SUCCESS)
 	{
+	  pam_end (pamh, ret);
+	  /* after using PAM we have to reset openlog data */
+	  openlog (program, LOG_CONS | LOG_ODELAY, LOG_DAEMON);
 	  dbg_log (_("User %s: %s"), account,
 		   pam_strerror (pamh, ret));
 	  sleep (getlogindefs_num ("FAIL_DELAY", 1));
@@ -716,6 +724,8 @@ cannot handle request version %d; current version is %d"),
 	  goto send_finish;
 	}
       pam_end (pamh, PAM_SUCCESS);
+      /* after using PAM we have to reset openlog data */
+      openlog (program, LOG_CONS | LOG_ODELAY, LOG_DAEMON);
 
       if (req->request == START_ADMIN)
 	{ /* In Admin mode send info text what we are now doing.  */
@@ -760,6 +770,8 @@ cannot handle request version %d; current version is %d"),
   ret = pam_start ("rpasswd", username, &conv, &pamh);
   if (ret != PAM_SUCCESS)
     {
+      /* after using PAM we have to reset openlog data */
+      openlog (program, LOG_CONS | LOG_ODELAY, LOG_DAEMON);
       dbg_log ("Couldn't initialize PAM: %s", pam_strerror (pamh, ret));
       retval = E_PAM_ERROR;
       goto send_finish;
@@ -769,6 +781,9 @@ cannot handle request version %d; current version is %d"),
   ret = pam_chauthtok (pamh, flags);
   if (ret != PAM_SUCCESS)
     {
+      pam_end (pamh, ret);
+      /* after using PAM we have to reset openlog data */
+      openlog (program, LOG_CONS | LOG_ODELAY, LOG_DAEMON);
       dbg_log (_("User %s: %s"), username,
 	       pam_strerror (pamh, ret));
       sleep (getlogindefs_num ("FAIL_DELAY", 1));
@@ -779,6 +794,9 @@ cannot handle request version %d; current version is %d"),
 
   pam_end (pamh, PAM_SUCCESS);
 
+  /* after using PAM we have to reset openlog data */
+  openlog (program, LOG_CONS | LOG_ODELAY, LOG_DAEMON);
+
   if (retval)
     send_string (ssl, TEXT_INFO, _("Password not changed"));
 #if 0
@@ -846,9 +864,10 @@ initialize_gnutls_session (const char *certificate, const char *privatekey)
    `poll' call makes sure only one thread handles an incoming connection.  */
 static void
 #ifdef USE_GNUTLS
-server_run (const char *certificate, const char *privatekey)
+server_run (const char *certificate, const char *privatekey,
+	    const char *program)
 #else
-server_run (void)
+server_run (const char *program)
 #endif
 {
   int i;
@@ -1099,7 +1118,8 @@ server_run (void)
 		case 0:
 		  /* Child: get all the data and process it.  */
 		  {
-		    int ret = handle_request (ssl, &req, locale, username);
+		    int ret = handle_request (ssl, &req, locale, username,
+					      program);
 
 		    close (fd);
 		    exit (ret);
@@ -1368,7 +1388,8 @@ main (int argc, char **argv)
 
       setsid ();
 
-      chdir ("/");
+      if (chdir ("/") < 0)
+	dbg_log ("chdir(\"/\") failed: %s", strerror (errno));
 
       openlog (program, LOG_CONS | LOG_ODELAY, LOG_DAEMON);
 
@@ -1420,9 +1441,9 @@ main (int argc, char **argv)
 
   /* Handle incoming requests.  */
 #ifdef USE_GNUTLS
-  server_run (certificate, privatekey);
+  server_run (certificate, privatekey, program);
 #else
-  server_run ();
+  server_run (program);
 #endif
 
   return E_SUCCESS;
