@@ -1,4 +1,4 @@
-/* Copyright (C) 2003, 2004 Thorsten Kukuk
+/* Copyright (C) 2003, 2004, 2005 Thorsten Kukuk
    Author: Thorsten Kukuk <kukuk@suse.de>
 
    This program is free software; you can redistribute it and/or modify
@@ -182,7 +182,8 @@ convert_grpopt_to_name (const char *arg, char **grouplist, gid_t *groupid,
 
       if (err == -1)		/* invalid number */
 	{
-	  fprintf (stderr, _("%s: Invalid numeric argument `%s'.\n"),
+	  fprintf (stderr,
+		   _("%s: Invalid numeric argument `%s' for group ID.\n"),
 		   program, arg);
 	  return E_BAD_ARG;
 	}
@@ -328,7 +329,7 @@ load_defaults (const char *configfile, struct default_t *data)
 	       && errno == ERANGE))	/* invalid number */
 	    {
 	      ++errors;
-	      fprintf (stderr, _("%s: Invalid numeric argument `%s' in %s.\n"),
+	      fprintf (stderr, _("%s: Invalid numeric argument `%s' for `INACTIVE' in %s.\n"),
 		       program, &cp[9], configfile);
 	    }
 	  else
@@ -503,8 +504,26 @@ write_defaults (const char *configfile, struct default_t *data)
       fclose (fp);
       return -1;
     }
-  fchmod (new_fd, oldmode.st_mode);
-  fchown (new_fd, oldmode.st_uid, oldmode.st_gid);
+  if (fchmod (new_fd, oldmode.st_mode) < 0)
+    {
+      fprintf (stderr,
+	       _("Cannot change mode for `%s': %s\n"),
+	       tmpname, strerror (errno));
+      fclose (fp);
+      close (new_fd);
+      unlink (tmpname);
+      return -1;
+    }
+  if (fchown (new_fd, oldmode.st_uid, oldmode.st_gid) < 0)
+    {
+      fprintf (stderr,
+	       _("Cannot change owner/group for `%s': %s\n"),
+	       tmpname, strerror (errno));
+      fclose (fp);
+      close (new_fd);
+      unlink (tmpname);
+      return -1;
+    }
   if (copy_xattr (configfile, tmpname) != 0)
     {
       fclose (fp);
@@ -881,6 +900,8 @@ static int
 create_home_directory (const char *home, uid_t uid, gid_t gid,
 		       const char *skeldir)
 {
+  int retval = 0;
+
   if (home == NULL || *home == '\0')
     return E_HOMEDIR;
 
@@ -910,13 +931,28 @@ create_home_directory (const char *home, uid_t uid, gid_t gid,
 			   program, path);
 		  return E_HOMEDIR;
 		}
-	      chown (path, 0, 0);
-	      chmod (path, 0755);
+	      if (chown (path, 0, 0) < 0)
+		fprintf (stderr, _("%s: Warning: chown on `%s' failed: %m\n"),
+			 program, path);
+	      if (chmod (path, 0755) < 0)
+		fprintf (stderr, _("%s: Warning: chmod on `%s' failed: %m\n"),
+			 program, path);
 	    }
 	  cp = strtok (NULL, "/");
 	}
-      chown (home, uid, gid);
-      chmod (home, 0777 & ~getlogindefs_num ("UMASK", 077));
+      if (chown (home, uid, gid) < 0)
+	{
+	  fprintf (stderr,
+		   _("Cannot change owner/group for `%s': %s\n"),
+		   home, strerror (errno));
+	  retval = E_HOMEDIR;
+	}
+      if (chmod (home, 0777 & ~getlogindefs_num ("UMASK", 077)) < 0)
+	{
+	  fprintf (stderr, _("Cannot change mode for `%s': %s\n"),
+		   home, strerror (errno));
+	  retval = E_HOMEDIR;
+	}
     }
 
   if (skeldir != NULL && *skeldir != '\0' && access (skeldir, F_OK) == 0)
@@ -925,11 +961,11 @@ create_home_directory (const char *home, uid_t uid, gid_t gid,
 	{
 	  fprintf (stderr, _("%s: Copying of skel directory failed.\n"),
 		   program);
-	  return -1;
+	  retval = E_HOMEDIR;
 	}
     }
 
-  return 0;
+  return retval;
 }
 
 int
@@ -1065,7 +1101,7 @@ main (int argc, char **argv)
 	    inactive = strtol (optarg, &cp, 10);
 	    if (*cp != '\0')	/* invalid number */
 	      {
-		fprintf (stderr, _("%s: Invalid numeric argument `%s'.\n"),
+		fprintf (stderr, _("%s: Invalid numeric argument `%s' for `-f'.\n"),
 			 program, optarg);
 		return E_BAD_ARG;
 	      }
@@ -1173,7 +1209,7 @@ main (int argc, char **argv)
 	  print_help (program);
 	  return 0;
 	case 'v':
-	  print_version (program, "2004");
+	  print_version (program, "2005");
 	  return 0;
 	case '\254':
 	  print_usage (stdout, program);
@@ -1303,6 +1339,9 @@ main (int argc, char **argv)
       /* -D option was given and we plan to modify
          entries in /etc/default/useradd.  */
 
+      if (new_home)
+	dflt.home = new_home;
+
       if (new_shell)
 	dflt.shell = new_shell;
 
@@ -1446,8 +1485,11 @@ main (int argc, char **argv)
 	      struct faillog fl;
 
 	      memset (&fl, 0, sizeof (fl));
-	      lseek (fd, (off_t) sizeof (fl) * pw_data.pw.pw_uid, SEEK_SET);
-	      write (fd, &fl, sizeof (fl));
+	      if ((lseek (fd, (off_t) sizeof (fl) * pw_data.pw.pw_uid, SEEK_SET)
+		   == (off_t)-1) || (write (fd, &fl, sizeof (fl)) == -1))
+		fprintf (stderr,
+			 _("%s: Error: Cannot clear old faillog entry: %s\n"),
+			 program, strerror (errno));
 	      close (fd);
 	    }
 
@@ -1456,8 +1498,11 @@ main (int argc, char **argv)
 	      struct lastlog ll;
 
 	      memset (&ll, 0, sizeof (ll));
-	      lseek (fd, (off_t) sizeof (ll) * pw_data.pw.pw_uid, SEEK_SET);
-	      write (fd, &ll, sizeof (ll));
+	      if ((lseek (fd, (off_t) sizeof (ll) * pw_data.pw.pw_uid, SEEK_SET)
+		   == (off_t)-1) || (write (fd, &ll, sizeof (ll)) == -1))
+		fprintf (stderr,
+			 _("%s: Error: Cannot clear old lastlog entry: %s\n"),
+			 program, strerror (errno));
 	      close (fd);
 	    }
 	}
