@@ -1,4 +1,4 @@
-/* Copyright (C) 2003, 2004 Thorsten Kukuk
+/* Copyright (C) 2003, 2004, 2005 Thorsten Kukuk
    Author: Thorsten Kukuk <kukuk@suse.de>
 
    This program is free software; you can redistribute it and/or modify
@@ -529,8 +529,12 @@ move_home_directory (const char *oldhome, const char *newhome)
 			   program, path);
 		  return E_HOMEDIR;
 		}
-	      chown (path, 0, 0);
-	      chmod (path, 0755);
+              if (chown (path, 0, 0) < 0)
+                fprintf (stderr, _("%s: Warning: chown on `%s' failed: %m\n"),
+                         program, path);
+              if (chmod (path, 0755) < 0)
+                fprintf (stderr, _("%s: Warning: chmod on `%s' failed: %m\n"),
+                         program, path);
 	    }
 	  cp = strtok (NULL, "/");
 	}
@@ -975,7 +979,7 @@ main (int argc, char **argv)
 	  print_help (program);
 	  return 0;
 	case 'v':
-	  print_version (program, "2004");
+	  print_version (program, "2005");
 	  return 0;
 	case '\254':
 	  print_usage (stdout, program);
@@ -1693,12 +1697,25 @@ main (int argc, char **argv)
 	{
 	  struct faillog fl;
 
-	  lseek (fd, (off_t) sizeof (fl) * pw_data->pw.pw_uid, SEEK_SET);
-	  if (read (fd, &fl, sizeof (fl)) == sizeof (fl))
+	  if (lseek (fd, (off_t) sizeof (fl) * pw_data->pw.pw_uid, SEEK_SET)
+	      == (off_t) -1)
 	    {
-	      lseek (fd, (off_t) sizeof (fl) * new_uid, SEEK_SET);
-	      write (fd, &fl, sizeof (fl));
+	      fprintf (stderr, _("%s: Error: Cannot copy faillog entry: %s\n"),
+		       program, strerror (errno));
+	      retval = E_FAILURE;
 	    }
+	  else
+	    if (read (fd, &fl, sizeof (fl)) == sizeof (fl))
+	      {
+		if ((lseek (fd, (off_t) sizeof (fl) * new_uid, SEEK_SET)
+		     == (off_t) -1) || (write (fd, &fl, sizeof (fl)) == -1))
+		  {
+		    fprintf (stderr,
+			     _("%s: Error: Cannot copy faillog entry: %s\n"),
+			    program, strerror (errno));
+		    retval = E_FAILURE;
+		  }
+	      }
 	  close (fd);
 	}
 
@@ -1706,13 +1723,26 @@ main (int argc, char **argv)
 	{
 	  struct lastlog ll;
 
-	  lseek (fd, (off_t) sizeof (ll) * pw_data->pw.pw_uid, SEEK_SET);
-	  if (read (fd, &ll, sizeof ll) == sizeof ll)
+	  if (lseek (fd, (off_t) sizeof (ll) * pw_data->pw.pw_uid, SEEK_SET)
+	      == (off_t) -1)
 	    {
-	      lseek (fd, (off_t) sizeof (ll) * new_uid, SEEK_SET);
-	      write (fd, &ll, sizeof (ll));
-	      close (fd);
+	      fprintf (stderr, _("%s: Error: Cannot copy lastlog entry: %s\n"),
+		       program, strerror (errno));
+	      retval = E_FAILURE;
 	    }
+	  else
+	    if (read (fd, &ll, sizeof ll) == sizeof ll)
+	      {
+		if ((lseek (fd, (off_t) sizeof (ll) * new_uid, SEEK_SET)
+		     == (off_t) -1) || (write (fd, &ll, sizeof (ll)) == -1))
+		  {
+		    fprintf (stderr,
+			     _("%s: Error: Cannot copy faillog entry: %s\n"),
+			     program, strerror (errno));
+		    retval = E_FAILURE;
+		  }
+	      }
+	  close (fd);
 	}
     }
   if (use_service == NULL || strcmp (use_service, "files") == 0)
