@@ -1,4 +1,4 @@
-/* Copyright (C) 2003, 2004 Thorsten Kukuk
+/* Copyright (C) 2003, 2004, 2005 Thorsten Kukuk
    Author: Thorsten Kukuk <kukuk@suse.de>
 
    This program is free software; you can redistribute it and/or modify
@@ -28,7 +28,6 @@
 #include <fcntl.h>
 #include <string.h>
 #include <unistd.h>
-#include <syslog.h>
 #include <signal.h>
 #include <getopt.h>
 #include <locale.h>
@@ -282,8 +281,22 @@ edit (const char *file, const char *program, int shadow)
       char *old;
 
       /* Set modes of temporary file to the from the original one.  */
-      chmod (tmpname, orig.st_mode);
-      chown (tmpname, orig.st_uid, orig.st_gid);
+      if (chmod (tmpname, orig.st_mode) < 0)
+	{
+	  fprintf (stderr,
+		   _("Cannot change mode for `%s': %s\n"),
+		   tmpname, strerror (errno));
+	  unlink (tmpname);
+	  return E_FAILURE;
+	}
+      if (chown (tmpname, orig.st_uid, orig.st_gid) < 0)
+	{
+	  fprintf (stderr,
+		   _("Cannot change owner/group for `%s': %s\n"),
+		   tmpname, strerror (errno));
+	  unlink (tmpname);
+	  return E_FAILURE;
+	}
 
       if (copy_xattr (file, tmpname) != 0)
 	{
@@ -300,7 +313,9 @@ edit (const char *file, const char *program, int shadow)
 	{
 	  /* Replace original file with edited one.  */
 	  unlink (old);
-	  link (file, old);
+	  if (link (file, old) < 0)
+	    fprintf (stderr,
+		     _("Warning: cannot create backup file: %m\n"));
 	  rename (tmpname, file);
 	}
     }
@@ -337,8 +352,6 @@ main (int argc, char *argv[])
       vipw = 1;
     }
 
-  openlog (program, LOG_PID, LOG_AUTHPRIV);
-
   while (1)
     {
       int c;
@@ -376,7 +389,7 @@ main (int argc, char *argv[])
           return 0;
 	case 'V': /* RH compatibility.  */
         case 'v':
-          print_version (program, "2004");
+          print_version (program, "2005");
           return 0;
         case 'u':
           print_usage (stdout, program);
